Print["----------------- Model: XYZ DM model ----------------"];
cardinfo                :=  {"-------- card: "<>parameters<>" mDM="<>ToString[N[mDM]]<>"  gDM="<>ToString[gDM]<>"--------"};
(* Values of mDM and gDM need to be specified in parameters.wl. Cardinfo is printed when using main.wls. *)


(* ---------------------------------------------- Default input   ----------------------------------------------*)

(* ---- annihilation  ---- *)

sv[s_]              := ; (* Enter here the annihilation cross section sigma*v_lab [GeV^-2] as a function of Mandelstam s *)
resonance           :=  {}; (* List containing position of annihilation resonances in Mandelstam s, see VRES.wl for example. Leave list empty if non. Sort values from smallest to
the largest and inside {s_min,infinity} if multiple resonances are present*)
threshold           :=  {}; (* List containing position of annihilation thresholds in Mandelstam s, see TH.wl for example. Leave list empty if non. *)

(* ---- scattering 1: Fokker-Planck approximation, PrepSCATT call redundant ---- *)

gam[x_]             := ; (* Enter here the momentum transfer rate \[Gamma](x) [GeV] as a function of x=mDM/T *)
scatttype           =   "gamma(x)";

(* ---- scattering 2: Fokker-Planck approximation, PrepSCATT call used to compute gam[x] ---- *)
(*
scatttype           =   "M2";
tM2sc[kt_]          := ; "If scatttype = M2, supply here the <|M|^2>_t for scattering as a fuction of \tilde k";
gamma[x_]           := ; "If scatttype = M2, supply here the integration routine to get scattering rate from <|M|^2>_t";
*)


(* ---------------------------------------------- Optional input   ----------------------------------------------*)

(* ---- Non-relativistic annihilation  ---- *)
(* For sufficiently smooth cross sections, where no sharp features like resonances or thresholds are present, it may be useful to adopt a non-relativistic treatment of the DM annihilation cross section and thermal averages. See SE.wl for example. Activated if RelThermalAv= False (in settings.wl). PrepANN and PrepANN2 then adopt non-relativistic thermal averages.
svnonrel[vrel_]      = ; (* Non-relativistic annihilation cross section sigma*v_lab [GeV^-2] as a function of relative velocity vlab (<<1, expanded) *)
resonance           := {};
threshold           := {};
*)

(*-----------------------------------------------------------------------------------------------------------------*)

(* The following optional routines for fBE and cBE Boltzmann solvers are C compiled to ensure short time access.
Compiled routines can not have access to global model parameters.
Therefore, required model parameters are (automatically) passed via the following one-dimensional list (arbitrary length) *)
(*
modelparameters:={mDM,(*insert other needed model parameters here*)}; (* modelparameters is passed to the below routines (called param inside) *)
*)

(* ---- < \sigma v >_theta analytic  ---- *)
(*
If the angular average of svmol can be computed analytically, it is recommended to insert that expression into GetAnnMatrixAnalytic (below).
GetAnnMatrixAnalytic usage is activated in fBE automatically if the setting variable Analyticsvtheta  = True;  PrepANNsvtheta call is then redundant.
*)
(*
GetAnnMatrixAnalytic=Compile[{{pref,_Real,0},{dim,_Integer,0},{c2,_Real,0},{xi,_Real,0},{tq,_Real,1},{param,_Real,1}},Module[{i,j,matrix,qi,qj,xqi,xqj,q,xq,mDM},
(*
INPUT:

    pref        =   [gDM/(2 Pi^2 tilde H x)]*(m/Sqrt[x])^3 dqA for momentum coordinates A,
    pref        =   [gDM/(2 Pi^2 tilde H x)]*(m/x)^3 g_s dqB for momentum coordinates B.
    dim         -   Dimesion of tq (momentum table)
    p/T         =   Sqrt[c2] tq, where c2 and tq depend on momentum coordinates A and B.
    xi          =   mDM/T
    param       =   modelparameters

OUTPUT:

pref*<svmol>_theta(p,tilde p) -- angular averaged annihilation cross section in form of a matrix
*)

matrix                          =   ConstantArray[0.,{dim,dim}]; (* = <svmol>_theta(p,tilde p) *)
q                               =   Sqrt[c2] tq; (*=p/T (always, independent of momentum coordinates)*)
xq                              =   Sqrt[xi^2+q^2]; (*=E/T*)

mDM = param[[1]];

(* Fill the matrix entries *)
Do[
    qi                          =   q[[i]];
    xqi                         =   xq[[i]];
    matrix[[i,i]]               = ;(*Insert <svmol>_theta(p_i,p_i) diagonal elements*)

    Do[
        qj                      =   q[[j]];
        xqj                     =   xq[[j]];
        matrix[[i,j]]           = ;(*Insert <svmol>_theta(p_i,p_j) off-diagonal elements*)
        matrix[[j,i]]           =   matrix[[i,j]];(*Matrix is symmetric*)

    ,{j,i+1,dim}
    ];

,{i,1,dim}
];

pref*matrix

], RuntimeOptions->"Speed", CompilationTarget->"C"];
*)


(* --------------------------------- C_el usage in fBE  ---------------------------------------- *)
(*
GetScatMatrix used in fBE if setting variable FullCell  = True;
*)
(*
GetScatMatrix=Compile[{{pref,_Real,0},{dim,_Integer,0},{c2,_Real,0},{xi,_Real,0},{tq,_Real,1},{param,_Real,1}},Module[{i,j,matrix,qi,qj,xqi,xqj,q,xq,inter,mDM},
(*
INPUT:

    pref        =   [gDM/(2 Pi^2 tilde H x)]*(m/Sqrt[x])^3 dqA for momentum coordinates A,
    pref        =   [gDM/(2 Pi^2 tilde H x)]*(m/x)^3 g_s dqB for momentum coordinates B.
    dim         -   Dimesion of tq
    tq          -   momentum table
    p/T         =   Sqrt[c2] tq, where c2 and tq depend on momentum coordinates A and B.
    xi          =   mDM/T
    param       =   modelparameters

OUTPUT:

pref*matrix     -   scattering collision term as a matrix


COMMENT:

matrix          =   \hat{C}_el * mDM^2 2 \Pi^2 /(c2 T^2 \Delta p)
\partial_x f    =   pref matrix.f + annihilation + ...
*)


matrix                              =   ConstantArray[0.,{dim,dim}];
q                                   =   Sqrt[c2] tq;
xq                                  =   Sqrt[xi^2+c2*tq^2];

mDM = param[[1]];

Do[

    qi                              =   q[[i]];
    xqi                             =   xq[[i]];

    Do[

        qj                          =   q[[j]];
        xqj                         =   xq[[j]];
        matrix[[i,j]]=q[[j]]^2(* !!! Insert mDM^2 <W>_{ij} here !!! *);
        matrix[[j,i]]=q[[i]]^2(* !!! Insert mDM^2 <W>_{ji} here !!! *);

    ,{j,i+1,dim}
    ];

,{i,1,dim}
];

(* Automatized construction of \hat C_el *)
Do[

    inter                           =   0.;

    Do[

        inter                       -=  matrix[[i,j]]Exp[(xq[[i]]-xq[[j]])/2.];

    ,{j,1,dim}
    ];

    matrix[[i,i]]                   =   inter;

,{i,1,dim}
];

Do[

    Do[
        matrix[[i,j]]               =   matrix[[i,j]] Exp[-((xq[[i]]-xq[[j]]))/2.];
        matrix[[j,i]]               =   matrix[[j,i]] Exp[((xq[[i]]-xq[[j]]))/2.];
    ,{j,i+1,dim}
    ];

,{i,1,dim}
];

pref*matrix

], RuntimeOptions->"Speed", CompilationTarget->"C"];
*)


(* --------------------------------- C_el usage in cBE  ---------------------------------------- *)

(*GetSecondMomentScat used in cBE if FullCell  = True;*)
(*
GetSecondMomentScat=Compile[{{xi,_Real,0},{yeq,_Real,0},{y,_Real,0},{ExpDbk2,_Real,0},{derbk2Dbk2,_Real,0},{param,_Real,1}},Module[{dim,i,j,matrix,yeqdy,xDM,qi,qj,xqi,xqj,q,xq,qmin,qmax,dq,fdm,avscatt,deravscatt,secmom,c,inter,mDM},
(*
INPUT:

    xi          =   mDM/T
    yeq         =   mDM T/s^(2/3)
    y           =   mDM T_chi/s^(2/3)
    ExpDbk2     =   Exp[-xDM]/BesselK2[xDM]
    derbk2Dbk2  =   BesselK2'[xDM]/BesselK2[xDM] -- with prime = d/dxDM
    param       =   modelparameters

OUTPUT:

{avscatt,deravscatt} -- second momentum moment average of scattering collision term and its derivative with respect to y

COMMENT:

avscatt         =   < C_el>_2 * T_chi/(m_chi^2)
deravscatt      =   d avscatt /dy
*)

(* construct moving q grid *)

dim                         =   250; (* incease for higher accuracy *)
yeqdy                       =   yeq/y;
qmin                        =   10^(-4)Sqrt[2.xi/yeqdy];
qmax                        =   8.*Sqrt[2. xi/yeqdy];
dq                          =   (qmax-qmin)/(dim-1.);
q                           =   Table[qmin+(k-1.) dq,{k,dim}];
xDM                         =   xi*yeqdy;
matrix                      =   ConstantArray[0.,{dim,dim}];
xq                          =   Sqrt[xi^2+q^2];
fdm                         =   Exp[-(yeqdy*xq-xDM)]*ExpDbk2;

mDM = param[[1]];

Do[

    qi                      =   q[[i]];
    xqi                     =   xq[[i]];

    Do[
        qj                  =   q[[j]];
        xqj                 =   xq[[j]];

        matrix[[i,j]]=q[[j]]^2(* !!! Insert mDM^2 <W>_{ij} here !!! *);
        matrix[[j,i]]=q[[i]]^2(* !!! Insert mDM^2 <W>_{ji} here !!! *);
    ,{j,i+1,dim}
    ];

,{i,1,dim}
];

(* Automatized construction of \hat C_el *)

Do[

    inter                   =   0.;

    Do[
        inter               -=  matrix[[i,j]]Exp[(xq[[i]]-xq[[j]])/2.];
    ,{j,1,dim}
    ];

    matrix[[i,i]]=inter;
,{i,1,dim}
];

Do[
    Do[
        matrix[[i,j]]       =   matrix[[i,j]] Exp[-((xq[[i]]-xq[[j]]))/2.];
        matrix[[j,i]]       =   matrix[[j,i]]Exp[((xq[[i]]-xq[[j]]))/2.];
    ,{j,i+1,dim}
    ];
,{i,1,dim}
];

(* Now matrix = \hat{C}_el * mDM^2 2 \Pi^2 /( T^2 \Delta p) *)

(* vectorized computation of second momentum moment average of scattering collision term and its derivative *)

c                           =   (xDM xi^(-7) dq^2/(6.*3.1415^2));
secmom                      =   q^4/xq;
avscatt                     =   c*((secmom).(matrix.fdm));
deravscatt                  =   ((xDM derbk2Dbk2) -1.)*(avscatt/y)+c*(yeqdy/y)*((secmom).(matrix.(xq*fdm)));

{avscatt,deravscatt}

], RuntimeOptions->"Speed", CompilationTarget->"C"];
*)
